using System;
using System.Collections.Generic;
using System.Text;

namespace PhysicsTestbed
{
    public class Angle
    {
		private Vector2 data;

		public Angle()
		{
			data = new Vector2(1, 0);
		}

        public Angle(double f)
        {
            data.X = Math.Cos(f);
            data.Y = Math.Sin(f);
        }

        public Angle(Vector2 v)
        {
			if (v == new Vector2(0, 0))
			{
				data = new Vector2(1, 0);
			}
			else
			{
				data = new Vector2(v.X, v.Y);
				data.Normalize();
			}
        }

        public double ToRadians()
        {
			double val = Math.Atan2(data.Y, data.X);
			if (val < 0) val += 2 * Math.PI;
			return val;
		}

		public double ToMinRadians()
		{
			double val = Math.Atan2(data.Y, data.X);
			if (val < -Math.PI) val += 2 * Math.PI;
			else if (val > Math.PI) val -= 2 * Math.PI;
			return val;
		}

		public Vector2 ToVector()
		{
			return data;
		}

		/*
		public Vector3 ToVector3()
		{
			return new Vector3(data.X, data.Y, 0f);
		}

		public Vector4 ToVector4()
		{
			return new Vector4(data.X, data.Y, 0f, 1f);
		}
		 */

        public Angle Rotate(double angle)
        {
            return new Angle(ToRadians() + angle);
        }

        public Angle Rotate(Vector2 angle)
        {
            return new Angle(ToRadians() + new Angle(angle).ToRadians());
        }

		public static double AbsoluteDifference(Angle angleA, Angle angleB)
		{
			// Maximum value returned should be PI

			double a, b;
			a = angleA.ToRadians() + (2 * Math.PI - angleB.ToRadians());
			if (a > 2 * Math.PI) a -= 2 * Math.PI;
			a = -a;

			// Turn clockwise
			b = angleB.ToRadians() - angleA.ToRadians();
			if (b < 0) b += 2 * Math.PI;

			if (Math.Abs(a) < Math.Abs(b)) return Math.Abs(a);
			else return Math.Abs(b);
		}

		public static double SignedDifference(Angle angleA, Angle angleB)
		{
			// Maximum value returned should be PI

			double a, b;
			a = angleA.ToRadians() + (2 * Math.PI - angleB.ToRadians());
			if (a > 2 * Math.PI) a -= 2 * Math.PI;
			a = -a;

			// Turn clockwise
			b = angleB.ToRadians() - angleA.ToRadians();
			if (b < 0) b += 2 * Math.PI;

			if (Math.Abs(a) < Math.Abs(b)) return a;
			else return b;
		}

		public override string ToString()
		{
			return "(X: " + data.X + ", Y: " + data.Y + ") = " + ToRadians() + "r";
		}

		public double RenderUpAngle()
		{
			return new Angle(this.ToRadians() + Math.PI * 0.5).ToRadians();
		}

		public Vector2 RotateVector(Vector2 vec)
		{
			// Returns the vector (vec) rotated by this angle

			Vector2 xVec = data;
			Vector2 yVec = new Vector2(-data.Y, data.X);
			return (vec.X * xVec + vec.Y * yVec);
		}
    }
}
