using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;

namespace PhysicsTestbed
{
	public class SparseMatrix<T> : IEnumerable<T>
	{
		protected T[,] data;
		protected int count = 0;

		public SparseMatrix(int width, int height)
		{
			data = new T[width, height];
		}

		public T this[int x, int y]
		{
			get
			{
				if (x >= 0 && x < data.GetLength(0) && y >= 0 && y < data.GetLength(1))
					return data[x, y];
				else
					return default(T);
			}
			set
			{
				if (value != null && data[x, y] == null)
				{
					count++;
				}
				else if (value == null && data[x, y] != null)
				{
					count--;
				}
				data[x, y] = value;
			}
		}

		public int Count
		{
			get
			{
				return count;
			}
		}

		public IEnumerator<T> GetEnumerator()
		{
			foreach (T t in data)
			{
				if (t != null)
				{
					yield return t;
				}
			}
		}

		IEnumerator IEnumerable.GetEnumerator()
		{
			return GetEnumerator();
		}
	}
}
